package com.sooka.auth.controller;

import com.sooka.auth.form.LoginBody;
import com.sooka.auth.form.RegisterBody;
import com.sooka.auth.service.SysLoginService;
import com.sooka.auth.util.RsaUtil;
import com.sooka.auth.util.SecretKeyBo;
import com.sooka.common.core.constant.CacheConstants;
import com.sooka.common.core.constant.SecurityConstants;
import com.sooka.common.core.domain.R;
import com.sooka.common.core.utils.JwtUtils;
import com.sooka.common.core.utils.StringUtils;
import com.sooka.common.redis.service.RedisService;
import com.sooka.common.security.auth.AuthUtil;
import com.sooka.common.security.service.TokenService;
import com.sooka.common.security.utils.SecurityUtils;
import com.sooka.system.api.RemoteUserService;
import com.sooka.system.api.model.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * token 控制
 *
 * @author sooka
 */
@RestController
public class TokenController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private SysLoginService sysLoginService;
    @Resource
    private RedisService redisService;


    /**
     * 获取秘钥接口
     * 前端请求后该方法会生成一对秘钥，分别为公钥和私钥
     * 将公钥返回给前端用于加密，私钥存入缓存（60s）用于后台解密
     * Pc端调用
     */
    @PostMapping("getRemoteSecretKey")
    @ResponseBody
    public R<?> getRemoteSecretKey() {
        String publicKey;
        try {
            SecretKeyBo bo = RsaUtil.genKeyPair();
            redisService.setCacheObject("remoteSecretKey", bo.getPrivateKey(), 60L, TimeUnit.SECONDS);
            publicKey = bo.getPublicKey();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isNotEmpty(publicKey)) {
            return R.ok(publicKey, "操作成功");
        }
        return R.fail(null,"操作失败");
    }

    /**
     * 获取秘钥接口
     * 前端请求后该方法会生成一对秘钥，分别为公钥和私钥
     * 将公钥返回给前端用于加密，私钥存入缓存（60s）用于后台解密
     * Pc端调用
     */
    @PostMapping("getSecretKey")
    public R<?> getSecretKey(HttpServletRequest request) {
        String sessionId = request.getSession().getId();
        return getSecretKey(sessionId);
    }


    /**
     * 改造获取秘钥接口
     * App、Pc端通用
     */
    public R<?> getSecretKey(String sessionId) {
        String publicKey;
        try {
            SecretKeyBo bo = RsaUtil.genKeyPair();
            redisService.setCacheObject(sessionId, bo.getPrivateKey(), 60L, TimeUnit.SECONDS);
            publicKey = bo.getPublicKey();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isNotEmpty(publicKey)) {
            return R.ok(publicKey, "操作成功");
        }
        return R.fail("数据异常，请稍后重试");
    }

    @PostMapping("login")
    public R<?> login(@RequestBody LoginBody form, HttpServletRequest request) {
        //解密密码字符串
        sysLoginService.decryptPassword(form, request);
        // 用户登录
//        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword());
        // 获取登录token
        return R.ok(login(form));
    }

    public Map login(LoginBody form) {
        // 用户登录
        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword());
        // 获取登录token
        Map map = tokenService.createToken(userInfo);
        LoginUser user = tokenService.getLoginUser(map.get("access_token").toString());
        map.put("userinfo", user);
        return map;
    }

    @DeleteMapping("logout")
    public R<?> logout(HttpServletRequest request) {
        String token = SecurityUtils.getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            String username = JwtUtils.getUserName(token);
            // 删除用户缓存记录
            AuthUtil.logoutByToken(token);
            // 记录用户退出日志
            sysLoginService.logout(username);
        }
        return R.ok();
    }

    @PostMapping("refresh")
    public R<?> refresh(HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (StringUtils.isNotNull(loginUser)) {
            // 刷新令牌有效期
            tokenService.refreshToken(loginUser);
            return R.ok();
        }
        return R.ok();
    }

    @PostMapping("register")
    public R<?> register(@RequestBody RegisterBody registerBody) {
        // 用户注册
        sysLoginService.register(registerBody.getUsername(), registerBody.getPassword());
        return R.ok();
    }

    /**
     * 根据token获取登录用户和权限等信息
     * @param request
     * @return
     */
    @PostMapping("selectLoginInfoByToken")
    public R<?> selectLoginInfoByToken(HttpServletRequest request) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("access_token" , SecurityUtils.getToken(request));
        map.put("expires_in" , CacheConstants.EXPIRATION);
        LoginUser user = tokenService.getLoginUser(map.get("access_token").toString());
        map.put("userinfo", user);
        return R.ok(map);
    }

    /**
     * 根据用户名密码获取token
     * @param request
     * @return
     */
    @PostMapping("getTokenByUsernameAndPassword")
    public R<?> getTokenByUsernameAndPassword(@RequestBody LoginBody form, HttpServletRequest request) {
        //解密密码字符串
        sysLoginService.decryptPassword(form, request);
        // 用户登录
        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword());
        // 获取登录token
        Map map = tokenService.createToken(userInfo);
        return R.ok(map);
    }


}
